/*

This software is open source under "The modified BSD license."

Copyright (c) 1998,2002 Sunflat

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer. 
* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution. 
* The name of the author may not be used to endorse or promote products
derived from this software without specific prior written permission. 

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

/******************************************************************************
 * SFCave for Palm
 * Ver.0.04
 *****************************************************************************/
 

#include <PalmOS.h>
#include <PalmCompatibility.h>
#include <Vga.h>
#include <Silk.h>
#include <SonyCLIE.h>
#include <PalmUtils.h>

#include "SFCaveRsc.h"


/***********************************************************************
 * Misc functions
 ***********************************************************************/

static void * GetObjectPtr(FormPtr frmP,UInt16 objectID)
{
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}

static int Rand(void) {
	return SysRandom(0);
}

static long min32(long a,long b) {
	if (a<b) return a; else return b;
}

static long max32(long a,long b) {
	if (a>b) return a; else return b;
}


/***********************************************************************
 * Games
 ***********************************************************************/

// event interval
#define EVENT_INTERVAL 5

// hardware butttons
#define BUTTON_NUM 6
char * gButtonStr[BUTTON_NUM] = {"PageUp","PageDown","DateBook","Address","ToDo","MemoPad"};
UInt16 gButtonCode[BUTTON_NUM] = {pageUpChr,pageDownChr,hard1Chr,hard2Chr,hard3Chr,hard4Chr};
UInt16 gButtonBitMask[BUTTON_NUM] = {keyBitPageUp,keyBitPageDown,keyBitHard1,keyBitHard2,keyBitHard3,keyBitHard4};

// levels
#define LEVEL_NUM 3
char * gLevelStr[LEVEL_NUM] = {"Easy","Normal","Hard"};

// preferences DB
#define DB_VERSION 3

typedef struct 
{
	int dbVersion;
	int level;
	int highScore[LEVEL_NUM];
	int button;
} TStarterPreference;

#define APP_FILE_CREATOR 'SFCv'
#define APP_PREF_ID 0x00
#define APP_PREF_VERSION 0x01

// backup of key parameters
UInt16 gKeyP1,gKeyP2,gKeyP3;
Boolean gKeyP4;


// scene states
typedef enum {SS_NONE,SS_TITLE,SS_GAME,SS_MISS} TSceneState;
typedef enum {Palm, Handera330, SonyClie, SonyClie5} MachineType;

MachineType mType;
UInt16      hrRefNum,hrSRefNum; // clie library reference number
Boolean 	SilkLibLoaded;

RGBColorType		rcolorRed, rcolorBlue, rcolorWhite, rcolorBlack;
Int16				gWidth, gHeight;

TSceneState gSceneState=SS_NONE;
int gSceneCount=0;

// VRAM
WinHandle gOffScreen;

// key state (pushed/released)
Boolean gKeyState=false;

MemHandle gTitleLogoHandle;

// static parameters
#define SCROLL_SPEED 4
#define ME_X 40

#define TITLE_BAR_HEIGHT 12
#define CAVE_Y_MIN (TITLE_BAR_HEIGHT+3)*256L
#define CAVE_Y_MAX 157*256L
#define MAP_SIZE ((160-ME_X)/SCROLL_SPEED-1)

// global variables (head 'g' means 'global')
int gScore=0;
int gHighScore[LEVEL_NUM]={0,0,0};
int gLevel=0,gControlButton=0;
int gPreviousBarPos=0;
Boolean is35;

// cave's y-pos & velocity-y
// 256 = 1dot
long gCaveY,gCaveVY;

// my y-pos & velocity-y (ribbon)
// 256 = 1dot
long gMeY,gMeVY;

// current map index
int gMapIndex;

// Map structure
typedef struct {
	// ('#' means wall)
	long caveY1,caveY2; // ####y1   y2####  
	long barY1,barY2;
	int barType; // 0:none  /  1: y1###y2  /  2: ###y1  y2###
} TMapElem;

// Map
TMapElem gMap[MAP_SIZE];

//
static void OnTitle(void);
static void OnGame(void);
static void OnMiss(void);


static inline Int16 gFntSetFont(Int16 fontID)
{
	if (mType == Handera330) {
		fontID = VgaBaseToVgaFont(fontID);
	}
	else if (mType >= SonyClie) {
		fontID += hrStdFont - stdFont;
	}
	if (mType >= SonyClie)
		return HRFntSetFont(hrRefNum, fontID);
	else return FntSetFont(fontID);
}

static void gWinDrawChars(const Char *chars, Int16 len, Int16 x, Int16 y) 
{
	if (mType >= SonyClie) {
		x <<= 1;
		y <<= 1;
	}
	else if (mType == Handera330) {
		x = x * 3 / 2;
		y = y * 3 / 2;
	}
	if (mType >= SonyClie)
		HRWinDrawChars(hrRefNum, chars, len, x, y);
	else 
		WinDrawChars(chars, len, x, y);
}

static void gWinDrawRectangleFrame (FrameType frame, RectangleType *rP)
{
	RectangleType lrp;
	
	if (mType >= SonyClie) {
		lrp.topLeft.x = rP->topLeft.x * 2;
		lrp.topLeft.y = rP->topLeft.y * 2;
		lrp.extent.x = rP->extent.x * 2;
		lrp.extent.y = rP->extent.y * 2;
	}
	else if (mType == Handera330) {
		lrp.topLeft.x = rP->topLeft.x * 3 / 2;
		lrp.topLeft.y = rP->topLeft.y * 3 / 2;
		lrp.extent.x = rP->extent.x * 3 / 2;
		lrp.extent.y = rP->extent.y * 3 / 2;
	} 
	else {
		lrp.topLeft.x = rP->topLeft.x;
		lrp.topLeft.y = rP->topLeft.y;
		lrp.extent.x = rP->extent.x;
		lrp.extent.y = rP->extent.y;
	}
	
	if (mType >= SonyClie)
		HRWinDrawRectangleFrame(hrRefNum, frame, &lrp);
	else 
		WinDrawRectangleFrame(frame, &lrp);
}

static void gWinInvertRectangle(RectangleType *rP, UInt16 cornerDiam)
{
	RectangleType lrp;
	
	if (mType >= SonyClie) {
		lrp.topLeft.x = rP->topLeft.x * 2;
		lrp.topLeft.y = rP->topLeft.y * 2;
		lrp.extent.x = rP->extent.x * 2;
		lrp.extent.y = rP->extent.y * 2;
	}
	else if (mType == Handera330) {
		lrp.topLeft.x = rP->topLeft.x * 3 / 2;
		lrp.topLeft.y = rP->topLeft.y * 3 / 2;
		lrp.extent.x = rP->extent.x * 3 / 2;
		lrp.extent.y = rP->extent.y * 3 / 2;
	} 
	else {
		lrp.topLeft.x = rP->topLeft.x;
		lrp.topLeft.y = rP->topLeft.y;
		lrp.extent.x = rP->extent.x;
		lrp.extent.y = rP->extent.y;
	}
		
	if (mType >= SonyClie)
		HRWinInvertRectangle(hrRefNum, &lrp, cornerDiam);
	else 
		WinInvertRectangle(&lrp, cornerDiam);
}

static void gWinEraseRectangle(RectangleType *rP, UInt16 cornerDiam)
{
	RectangleType lrp;
	
	if (mType >= SonyClie) {
		lrp.topLeft.x = rP->topLeft.x * 2;
		lrp.topLeft.y = rP->topLeft.y * 2;
		lrp.extent.x = rP->extent.x * 2;
		lrp.extent.y = rP->extent.y * 2;
	}
	else if (mType == Handera330) {
		lrp.topLeft.x = rP->topLeft.x * 3 / 2;
		lrp.topLeft.y = rP->topLeft.y * 3 / 2;
		lrp.extent.x = rP->extent.x * 3 / 2;
		lrp.extent.y = rP->extent.y * 3 / 2;
	} 
	else {
		lrp.topLeft.x = rP->topLeft.x;
		lrp.topLeft.y = rP->topLeft.y;
		lrp.extent.x = rP->extent.x;
		lrp.extent.y = rP->extent.y;
	}
		
	if (mType >= SonyClie)
		HRWinEraseRectangle(hrRefNum, &lrp, cornerDiam);
	else 
		WinEraseRectangle(&lrp, cornerDiam);
}

static void gWinFillRectangle(RectangleType *rP, UInt16 cornerDiam)
{
	RectangleType lrp;
	
	if (mType >= SonyClie) {
		lrp.topLeft.x = rP->topLeft.x * 2;
		lrp.topLeft.y = rP->topLeft.y * 2;
		lrp.extent.x = rP->extent.x * 2;
		lrp.extent.y = rP->extent.y * 2;
	}
	else if (mType == Handera330) {
		lrp.topLeft.x = rP->topLeft.x * 3 / 2;
		lrp.topLeft.y = rP->topLeft.y * 3 / 2;
		lrp.extent.x = rP->extent.x * 3 / 2;
		lrp.extent.y = rP->extent.y * 3 / 2;
	} 
	else {
		lrp.topLeft.x = rP->topLeft.x;
		lrp.topLeft.y = rP->topLeft.y;
		lrp.extent.x = rP->extent.x;
		lrp.extent.y = rP->extent.y;
	}
		
	if (mType >= SonyClie)
		HRWinFillRectangle(hrRefNum, &lrp, cornerDiam);
	else 
		WinFillRectangle(&lrp, cornerDiam);
}

static void gWinDrawChar(WChar theChar, Int16 x, Int16 y) 
{
	if (mType >= SonyClie) {
		x <<= 1;
		y <<= 1;
	}
	else if (mType == Handera330) {
		x = x * 3 / 2;
		y = y * 3 / 2;
	}
	if (mType >= SonyClie)
		HRWinDrawChar(hrRefNum, theChar, x, y);
	else 
		WinDrawChar(theChar, x, y);
}


static void gWinDrawInvertedChars(const Char *chars, Int16 len, Int16 x, Int16 y) 
{
	if (mType >= SonyClie) {
		x <<= 1;
		y <<= 1;
	}
	else if (mType == Handera330) {
		x = x * 3 / 2;
		y = y * 3 / 2;
	}
	if (mType >= SonyClie)
		HRWinDrawInvertedChars(hrRefNum, chars, len, x, y);
	else 
		WinDrawInvertedChars(chars, len, x, y);
}

static void gWinDrawLine (Int16 x1, Int16 y1, Int16 x2, Int16 y2) 
{
	if (mType >= SonyClie) {
		x1 <<= 1; y1 <<= 1;
		x2 <<= 1; y2 <<= 1;
	}
	else if (mType == Handera330) {
		x1 = x1 * 3 / 2;
		y1 = y1 * 3 / 2;
		x2 = x2 * 3 / 2;
		y2 = y2 * 3 / 2;
	}
	if (mType >= SonyClie)
		HRWinDrawLine(hrRefNum, x1, y1, x2, y2);
	else
		WinDrawLine(x1, y1, x2, y2);
}

static void gWinEraseLine (Int16 x1, Int16 y1, Int16 x2, Int16 y2) 
{
	if (mType >= SonyClie) {
		x1 <<= 1; y1 <<= 1;
		x2 <<= 1; y2 <<= 1;
	}
	else if (mType == Handera330) {
		x1 = x1 * 3 / 2;
		y1 = y1 * 3 / 2;
		x2 = x2 * 3 / 2;
		y2 = y2 * 3 / 2;
	}
	if (mType >= SonyClie)
		HRWinEraseLine(hrRefNum, x1, y1, x2, y2);
	else
		WinEraseLine(x1, y1, x2, y2);
}

static void gWinCopyRectangle (WinHandle srcWin, WinHandle dstWin, 
	RectangleType *srcRect, Int16 destX, Int16 destY, WinDrawOperation mode) 
{
	RectangleType rect;
	if (mType >= SonyClie) {
		rect.topLeft.x = srcRect->topLeft.x * 2;
		rect.topLeft.y = srcRect->topLeft.y * 2;
		rect.extent.x = srcRect->extent.x * 2;
		rect.extent.y = srcRect->extent.y * 2;
		destX <<= 1;
		destY <<= 1;
	}
	else if (mType == Handera330) {
		rect.topLeft.x = srcRect->topLeft.x * 3 / 2;
		rect.topLeft.y = srcRect->topLeft.y * 3 / 2;
		rect.extent.x = srcRect->extent.x * 3 / 2;
		rect.extent.y = srcRect->extent.y * 3 / 2;
		
		destX = destX * 3 / 2;
		destY = destY * 3 / 2;
	}
	else {
		rect.topLeft.x = srcRect->topLeft.x;
		rect.topLeft.y = srcRect->topLeft.y;
		rect.extent.x = srcRect->extent.x;
		rect.extent.y = srcRect->extent.y;
	}
		
	if (mType >= SonyClie) {
		HRWinCopyRectangle (hrRefNum, srcWin, dstWin, &rect, 
			destX, destY, mode);
	}
	else {
		WinCopyRectangle (srcWin, dstWin, &rect, 
			destX, destY, mode);
	}
}

static void gWinDrawBitmap(BitmapPtr bitmapPtr, Int16 x, Int16 y)
{
	if (mType >= SonyClie) {
		x <<= 1; y <<= 1;
	}
	else if (mType == Handera330) {
		x = x * 3 / 2; y = y * 3 / 2;
	}
	
	if (mType >= SonyClie) {
		HRWinDrawBitmap(hrRefNum, bitmapPtr, x, y);
	}
	else {
		WinDrawBitmap(bitmapPtr, x, y);
	}
}

// set scene
static void SetSceneState(TSceneState a)
{
	gSceneState=a;
	gSceneCount=0; 
}

// init game
static void GameInit(void)
{
	gScore=0;
	SetSceneState(SS_TITLE);
}

// scene step (called periodically)
static void SceneStep(void)
{
	gSceneCount++;
	gKeyState=(Boolean)((KeyCurrentState() & gButtonBitMask[gControlButton])!=0);
	
	switch(gSceneState){
	case SS_TITLE:
		OnTitle();
		break;
	case SS_GAME:
		OnGame();
		break;
	case SS_MISS:
		OnMiss();
		break;
	}
}

static inline Int16 gFntCharWidth(Char ch)
{
	if (mType == SonyClie5) 
		return HRFntCharWidth(hrRefNum, ch);
	else
		return FntCharWidth(ch);
}


static inline Int16 gFntCharsWidth(const Char *str, Int16 len)
{
	if (mType == SonyClie5) return HRFntCharsWidth(hrRefNum, str, len);
	else return FntCharsWidth(str, len);
}


static void DrawStrCenter(char const *s,int x,int y)
{
	Int16 l=StrLen(s);
	Int16 width = gFntCharsWidth(s,l); 
	
	if (mType >= SonyClie) width >>= 1;
	else if (mType == Handera330) {
		width = width * 2 / 3;
	}
	
	gWinDrawChars(s,l,x - width/2,y);
}

// game scene step
static void OnGame(void)
{
	int i;
	int barHeightMin=0;
	long barHeight=0;
	int barType=0;
	long caveHeight;
	long oldMeY;
	int caveVY_max=0;
	TMapElem *m;
	int caveChangeInterval=0,gravitation=0;
	
	char buff[32];
	WinHandle tmpHandle;
	RectangleType rect;
	CustomPatternType brush;
	

	///	
	
	if (gSceneCount==1) {
		// initialize
		gCaveY=26*256L;gCaveVY=0;
		gMeY=60*256L;gMeVY=-512;
		gMapIndex=0;
		for(i=0;i<MAP_SIZE;i++) {
			m=&(gMap[i]);
			m->caveY1=CAVE_Y_MIN; m->caveY2=CAVE_Y_MAX;
			m->barY1=0; m->barY2=0;
			m->barType=0;
		}
		
		gScore=0;
		gPreviousBarPos=0;
	}

	// increment score
	gScore++;

	// cource parameters for each level
	switch(gLevel) {
	default:
		caveHeight=120L*700L*256L/(gScore+700L)+2*256L;
		caveVY_max=512;
		caveChangeInterval=8;
		gravitation=128;
		
		barType=0;
		if (gScore-gPreviousBarPos>20 && (Rand()%2)==0) {
			gPreviousBarPos=gScore;
			barType=1;
			barHeight=min32(20*256L,caveHeight*3/4);
			barHeightMin=8*256;
		}
		break;
		
	case 1:
		caveHeight=80L*600L*256L/(gScore+600L)+2*256L;
		caveVY_max=512+256;
		caveChangeInterval=8;
		gravitation=128;
		
		barType=0;
		if (gScore-gPreviousBarPos>20 && (Rand()%2)==0) {
			gPreviousBarPos=gScore;
			barType=2;
			barHeight=min32(20*256L,caveHeight*3/4);
			barHeightMin=4*256;
		}
		break;
		
	case 2:
		caveHeight=45L*400L*256L/(gScore+400L)+2*256L;
		caveVY_max=1024;
		caveChangeInterval=8;
		gravitation=128;
		barType=0;
		break;
	}
	

	// next step
	gMapIndex=(gMapIndex+1)%MAP_SIZE;
	m=&(gMap[gMapIndex]);

	oldMeY=gMeY;
	gMeVY+=gKeyState?-gravitation:gravitation;
	gMeY+=gMeVY;
	
	// miss?
	if ((gMeY<m->caveY1 || m->caveY2<gMeY)
		|| (m->barType==1 &&  m->barY1<gMeY && gMeY<m->barY2) 
		|| (m->barType==2 &&  (gMeY<m->barY1 || m->barY2<gMeY))
	){
		SetSceneState(SS_MISS); // bom!
	}

	// make map
	gCaveY+=gCaveVY;
	if (gCaveY<CAVE_Y_MIN){
		gCaveY=CAVE_Y_MIN;
		gCaveVY=Rand()%caveVY_max;
	}
	if (gCaveY>CAVE_Y_MAX-caveHeight){
		gCaveY=CAVE_Y_MAX-caveHeight;
		gCaveVY=-(Rand()%caveVY_max);
	}
	if (0==(Rand()%caveChangeInterval)) {
		gCaveVY=Rand()%(caveVY_max*2)-caveVY_max;
	}
	
	m->caveY1=gCaveY; m->caveY2=gCaveY+caveHeight;
	m->barY1=0; m->barY2=0;
	m->barType=0;
	
	if (barType==1){
		long bh=barHeight;
		if (bh>caveHeight-barHeightMin) bh=caveHeight-barHeightMin;
		if (barHeightMin<bh) {
			m->barY1=gCaveY+Rand()%(caveHeight-bh);
			m->barY2=m->barY1+bh;
			m->barType=barType;
		}
	}
	if (barType==2){
		long bh=caveHeight-barHeight;
		if (barHeightMin<bh && bh<caveHeight) {
			m->barY1=gCaveY+Rand()%(caveHeight-bh);
			m->barY2=m->barY1+bh;
			m->barType=barType;
		}
	}
	
	// draw ////////////
	
	tmpHandle = WinSetDrawWindow(gOffScreen);
	
	for(i=0;i<8;i++) brush[i]=(unsigned char)((Rand()|Rand()|Rand())&255);
	WinSetPattern(&brush);
	
	if (gSceneCount==1) {
		// initialize
		WinEraseWindow();
	}
		
	// scroll
	rect.topLeft.x = SCROLL_SPEED; rect.topLeft.y = TITLE_BAR_HEIGHT;
	rect.extent.x = 160-SCROLL_SPEED ; rect.extent.y = 160;

	gWinCopyRectangle(gOffScreen,gOffScreen,&rect,0,TITLE_BAR_HEIGHT,winPaint);
	
	rect.topLeft.x = 160-SCROLL_SPEED; rect.topLeft.y = TITLE_BAR_HEIGHT;
	rect.extent.x = SCROLL_SPEED ; rect.extent.y = 160;
	gWinEraseRectangle(&rect,0);

	if (is35) {
	    WinPushDrawState();
	}
	
	if (is35) {
		WinSetForeColor(WinRGBToIndex(&rcolorRed));
	}

	// draw bar
	if (m->barType==1){
		rect.topLeft.x=160-SCROLL_SPEED*2;rect.extent.x=SCROLL_SPEED*2;
		rect.topLeft.y = (int)(m->barY1/256L);
		rect.extent.y = (int)(m->barY2/256L) - rect.topLeft.y;
		
		for(i=0;i<8;i++) brush[i]|=(unsigned char)((Rand())&255);
		WinSetPattern(&brush);
		
		gWinFillRectangle(&rect,0);
	}
	if (m->barType==2){
		rect.topLeft.x=160-SCROLL_SPEED*2;rect.extent.x=SCROLL_SPEED*2;
		rect.topLeft.y = TITLE_BAR_HEIGHT;
		rect.extent.y = (int)(m->barY1/256L) - rect.topLeft.y;
		gWinFillRectangle(&rect,0);
		rect.topLeft.y = (int)(m->barY2/256L);
		rect.extent.y = 160 - rect.topLeft.y;
		gWinFillRectangle(&rect,0);
	}
	
	if (is35) {
		WinSetForeColor(WinRGBToIndex(&rcolorBlue));
	}
		
	// draw next wall
	rect.topLeft.y = TITLE_BAR_HEIGHT;
	rect.extent.y = (int)(m->caveY1/256L - rect.topLeft.y);
	gWinFillRectangle(&rect,0);
	rect.topLeft.y = (int)(m->caveY2/256L);
	rect.extent.y = 160 - rect.topLeft.y;
	gWinFillRectangle(&rect,0);
	

	if (is35) {
		WinPopDrawState();
	}

	if (is35) {
	    WinPushDrawState();
    	WinSetForeColor(WinRGBToIndex(&rcolorBlack));
	}
	// draw ribbon
	for (i=-2;i<=2;i++) gWinDrawLine(ME_X-SCROLL_SPEED+i,(int)(oldMeY/256L),ME_X+i,(int)(gMeY/256L));
	
	if (is35) {
		WinPopDrawState();
	}
	
	// paint
	rect.topLeft.x=0; rect.topLeft.y=TITLE_BAR_HEIGHT;
	rect.extent.x = 160; rect.extent.y = 160 - TITLE_BAR_HEIGHT;
	gWinCopyRectangle(gOffScreen,WinGetDisplayWindow(),&rect,0,TITLE_BAR_HEIGHT,winPaint);
	StrPrintF(buff," %d ",gScore);
	WinSetDrawWindow(WinGetDisplayWindow());
	gWinDrawInvertedChars(buff,(short)StrLen(buff),2,148);
	
	WinSetDrawWindow(tmpHandle);
}

// miss scene step
static void OnMiss(void)
{
	int i;
	static Boolean bPushed; // for button release check
	
	char buff[32];
	WinHandle tmpHandle;
	RectangleType rect;

	//
		
	tmpHandle = WinSetDrawWindow(gOffScreen);
	
	if (gSceneCount==1) {
		// initialize
		SndPlaySystemSound(sndWarning); // play sound
		bPushed=true;
		
		if (gMeY<0) gMeY=0;
		if (gMeY>158*256L) gMeY=158*256L;
	}
	
	bPushed&=gKeyState;
	
	if (gSceneCount<30) {
		// bom!
		if (is35) {
	    	WinPushDrawState();
		}
		for(i=0;i<5;i++){
			if (is35)
				WinSetForeColor(Rand()%255);
			gWinDrawLine(ME_X,(int)(gMeY/256L),ME_X+Rand()%(gSceneCount*8)-gSceneCount*4
				,(int)(gMeY/256L)+Rand()%(gSceneCount*8)-gSceneCount*4);
			gWinEraseLine(ME_X,(int)(gMeY/256L),ME_X+Rand()%(gSceneCount*4)-gSceneCount*2
				,(int)(gMeY/256L)+Rand()%(gSceneCount*4)-gSceneCount*2);
		}
		if (is35) {
	    	WinPushDrawState();
    		WinSetForeColor(WinRGBToIndex(&rcolorBlack));
		}
	}
	if (gSceneCount==30) {
		// draw gameover message
		char buff[32];
		RectangleType rect;
		rect.topLeft.x=40;rect.topLeft.y=40;rect.extent.x=160-80;rect.extent.y=160-80;
		gWinEraseRectangle(&rect,0);
		gWinDrawRectangleFrame(popupFrame,&rect);
		
		gFntSetFont(largeFont);
		gWinDrawInvertedChars(" GameOver ",10,48,50);
		gFntSetFont(boldFont);
		StrPrintF(buff," Score : %d ",gScore);
		DrawStrCenter(buff,80,85);
		if (gHighScore[gLevel]<gScore) DrawStrCenter(" High Score !! ",80,105);
		
	}
	
	if ((gSceneCount>35 && !bPushed && gKeyState) || gSceneCount==100) {
		// back to title
		if (gHighScore[gLevel]<gScore) gHighScore[gLevel]=gScore;
		SetSceneState(SS_TITLE);
	}
	
	// paint
	rect.topLeft.x=0; rect.topLeft.y=TITLE_BAR_HEIGHT;
	rect.extent.x = 160; rect.extent.y = 160 - TITLE_BAR_HEIGHT;
	gWinCopyRectangle(gOffScreen,WinGetDisplayWindow(),&rect,0,TITLE_BAR_HEIGHT,winPaint);
	StrPrintF(buff," %d ",gScore);
	WinSetDrawWindow(WinGetDisplayWindow());
	gWinDrawInvertedChars(buff,(short)StrLen(buff),2,148);
	
	WinSetDrawWindow(tmpHandle);
}

// title scene step
static void OnTitle(void)
{
	BitmapPtr bitmap;
	static int logoX;
	static Boolean bPushed; // for button release check
	char buff[64];
	
	WinHandle tmpHandle;
	RectangleType rect;
	
	//
	tmpHandle = WinSetDrawWindow(gOffScreen);
	
	if (gSceneCount==1) { 
		// initalize
		WinEraseWindow();
		logoX=160;
		bPushed=true;
	}
	else if (gSceneCount < 15) {
		WinEraseWindow();
	}
	
	bPushed&=gKeyState;
	
	if (gSceneCount==15) {
		
		// draw title messages
		gFntSetFont(boldFont);
		
		StrPrintF(buff,"Level : %s",(gLevel==0?"Easy":gLevel==1?"Normal":gLevel==2?"Hard":"?") );
		DrawStrCenter(buff,80,85);

		StrPrintF(buff,"High Score : %d",gHighScore[gLevel]);
		DrawStrCenter(buff,80,100);
		if (gScore!=0){
			StrPrintF(buff,"Last Score : %d",gScore);
			DrawStrCenter(buff,80,115);
		}
		
		StrPrintF(buff,"push %s button",gButtonStr[gControlButton]);
		DrawStrCenter(buff,80,134);
	}
	
	if (gSceneCount>15 && !bPushed && gKeyState) {
		// game start!
		SndPlaySystemSound(sndClick);
		SetSceneState(SS_GAME);
	}
	
	// draw title
	bitmap = MemHandleLock(gTitleLogoHandle);
	if (mType == Palm) {
		logoX=(logoX-10)*3/4+10;
	}
	else if (mType >= SonyClie) {
		logoX=(logoX)*3/4 + 10;
	}
	else if (mType == Handera330) {
		logoX = (logoX-4) * 3 / 4 + 10;	
	}
	gWinDrawBitmap(bitmap,logoX,24);
	MemHandleUnlock(gTitleLogoHandle);
	
	// paint
	rect.topLeft.x=0; rect.topLeft.y=TITLE_BAR_HEIGHT;
	rect.extent.x = 160; rect.extent.y = 160 - TITLE_BAR_HEIGHT;
	gWinCopyRectangle(gOffScreen,WinGetDisplayWindow(),&rect,0,TITLE_BAR_HEIGHT,winPaint);
	
	WinSetDrawWindow(tmpHandle);
}



/***********************************************************************
 * MainFormInit
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
	if (mType == Handera330) {
		VgaFormModify(frmP, vgaFormModify160To240);
		// SilkMinimizeWindow();
	}
	
	if (mType >= SonyClie) 
		HRWinGetDisplayExtent(hrRefNum, &gWidth, &gHeight);
	else 
		WinGetDisplayExtent(&gWidth, &gHeight);
		
	gTitleLogoHandle=DmGetResource('Tbmp',AppTitleBitmap);
	GameInit();
}

/***********************************************************************
 * DoOptionDialog
 ***********************************************************************/
static void DoOptionDialog()
{
	FormPtr frmP;
	
	ListPtr lstLevel,lstButton;
	ControlPtr trgLevel,trgButton;
	UInt16 ret;
	
	MenuEraseStatus(0);	// Clear the menu status from the display.
	
	// setup
	frmP = FrmInitForm (OptionForm);
	
	if (mType == Handera330) {
		VgaFormModify(frmP, vgaFormModify160To240);
		// SilkMinimizeWindow();
	}
	
	// level
	lstLevel=GetObjectPtr(frmP, OptionLevelList);
	LstSetListChoices(lstLevel,gLevelStr,LEVEL_NUM);
	LstSetHeight(lstLevel,LEVEL_NUM);
	LstSetSelection(lstLevel,gLevel);
	trgLevel=GetObjectPtr(frmP, OptionLevelPopTrigger);
	CtlSetLabel(trgLevel,gLevelStr[gLevel]);
	
	// button
	lstButton=GetObjectPtr(frmP, OptionButtonList);
	LstSetListChoices(lstButton,gButtonStr,BUTTON_NUM);
	LstSetHeight(lstButton,BUTTON_NUM);
	LstSetSelection(lstButton,gControlButton);
	trgButton=GetObjectPtr(frmP, OptionButtonPopTrigger);
	CtlSetLabel(trgButton,gButtonStr[gControlButton]);
	
	// show dialog
	WinEraseWindow();
	ret=FrmDoDialog(frmP);
	if (ret==OptionOKButton) {
		ControlPtr checkClear;
		int a;
	
		// apply options
		a=LstGetSelection(lstLevel);
		if (a>=0 && a<LEVEL_NUM) gLevel=a;
		a=LstGetSelection(lstButton);
		if (a>=0 && a<BUTTON_NUM) gControlButton=a;
		
		// clear high score?
		checkClear=GetObjectPtr(frmP, OptionClearScoreCheckbox);
		if (CtlGetValue(checkClear)!=0) {
			if (FrmAlert(ClearScoreAlert)==ClearScoreOK) {
				gHighScore[gLevel]=0;
			}
		}
		
		//
		GameInit();
	}
	
	FrmDeleteForm (frmP);
	
	// repaint owner
	frmP = FrmGetActiveForm();
	FrmDrawForm ( frmP);
}

/***********************************************************************
 * MainFormHandleEvent
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) 
	{
		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;
			
		case nilEvent:
			SceneStep();
			handled=true;
			break;
					
		case keyDownEvent:
		{
			WChar chr;
			
			chr=eventP->data.keyDown.chr;
			if (EvtKeydownIsVirtual(eventP) && (chr==vchrMenu)) {
				DoOptionDialog();		
			}
			
			handled=true;

			break;

		}
		default:
			break;
		
	}
	
	return handled;
}


/***********************************************************************
 * AppHandleEvent
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 * AppEventLoop
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, EVENT_INTERVAL /*evtWaitForever*/ );

		// disable Sys/MenuHandleEvent for hardware keys
		if ( event.eType == keyDownEvent && (event.data.keyDown.chr==gButtonCode[gControlButton])) {
			// forward to AppHandleEvent directly
			if (! AppHandleEvent(&event))
				FrmDispatchEvent(&event);
			
		}else {
		
			// normal event handling
			if (! SysHandleEvent(&event))
				if (! MenuHandleEvent(0, &event, &error))
					if (! AppHandleEvent(&event))
						FrmDispatchEvent(&event);
		}

	} while (event.eType != appStopEvent);
}

static void SetScreenMode(UInt32 depth)
{
	// screen setting
	if (mType == Palm || mType == Handera330) {
		if (is35) {
			WinScreenMode(winScreenModeSet, NULL, NULL, &depth, NULL);
	    }
	    else 
	    	ScrDisplayMode (winScreenModeSet, NULL, NULL, &depth, NULL);
	}
	else {
		UInt32      width, height;
		width = hrWidth; height = hrHeight;
	    HRWinScreenMode(hrRefNum, winScreenModeSet, &width, &height, &depth, NULL);
	}
}


/***********************************************************************
 *  AppStart
 ***********************************************************************/
static Err AppStart(void)
{
    TStarterPreference prefs;
    UInt16 prefsSize;
	UInt16 err,newKeyP1=255,newKeyP3=255;
	Boolean newKeyP4=false;
	Int16 	error;
	UInt32  version;
	UInt32	romversion, depth;
	SonySysFtrSysInfoP sonySysFtrSysInfoP;
	UInt16  refNum;
	UInt16	maxDepth;
	Int16	exx, exy;
	Boolean iscolor;
	
	// reduce key repeat
	KeyRates(false,&gKeyP1,&gKeyP2,&gKeyP3,&gKeyP4);
	KeyRates(true,&newKeyP1,&newKeyP3,&gKeyP3,&newKeyP4);
	
	// randomize
	SysRandom((long)TimGetSeconds());
	
	// Read the saved preferences / saved-state information.
	MemSet(&prefs,sizeof(prefs),0);
	prefsSize = sizeof(TStarterPreference);
	if (PrefGetAppPreferences(APP_FILE_CREATOR, APP_PREF_ID, &prefs, &prefsSize, true) != 
		noPreferenceFound && prefs.dbVersion<=DB_VERSION)
	{
		int i;
		if (prefs.dbVersion>=3) {
			int a;
			a=prefs.level;
			if (0<=a && a<LEVEL_NUM) gLevel=a;
			a=prefs.button;
			if (0<=a && a<BUTTON_NUM) gControlButton=a;
			for(i=0;i<LEVEL_NUM;++i) {
				gHighScore[i]=prefs.highScore[i];
			}
		}
	}
	
	error = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);
	is35 = (romversion >= 0x03503000);
	
	
  	//read system configuration
  	if (is35)
    {
	   	WinScreenMode(winScreenModeGetSupportsColor,NULL,NULL,NULL,&iscolor);
      	WinScreenMode(winScreenModeGetSupportedDepths,NULL,NULL,&depth,NULL);
    }
    else {
    	ScrDisplayMode(winScreenModeGetSupportedDepths, NULL, NULL, &depth, NULL);
    
    }

   	if ((depth & 32768) == 32768) maxDepth=8;
	else if ((depth & 128) == 128) maxDepth=8;
	else if ((depth & 8) == 8) maxDepth=4;
	else if ((depth & 2) == 2) maxDepth=2;
    else maxDepth=1;            
	
	if (!iscolor) maxDepth = 1;
	
	if (_TRGVGAFeaturePresent(&version)) mType = Handera330;
	else if (!FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &version)) {
        Err         error;   
        UInt16		ver;

        if (version == sonyHwrOEMCompanyID_Sony) {
       		if ((error = FtrGet(sonySysFtrCreator,
               	 sonySysFtrNumSysInfoP, (UInt32*)&sonySysFtrSysInfoP))) {
                // NOT CLIE
            }
            else {
                if ((sonySysFtrSysInfoP->extn & sonySysFtrSysInfoExtnSilk) &&
                    (sonySysFtrSysInfoP->libr & sonySysFtrSysInfoLibrSilk)) {
                    if ((error = SysLibFind(sonySysLibNameSilk, &refNum))) {
                        if (error == sysErrLibNotFound) {
                            // coulnd't find lib
                            error = SysLibLoad('libr', sonySysFileCSilkLib, &refNum);
                        }
                    }
                    if (!error) {
                        if (SilkLibOpen(refNum) == errNone) {
                            // SilkLibEnableResize(refNum);
                            hrSRefNum = refNum;
                            SilkLibLoaded = true;
                        }
                    }
                }
            }
            error = SysLibLoad('libr', sonySysFileCHRLib, &hrRefNum);
            
            if (!error) {
                HROpen(hrRefNum);

                HRGetAPIVersion(hrRefNum, &ver);
                if (ver >= HR_VERSION_SUPPORT_FNTSIZE) {
                	mType = SonyClie5;
               	}
                else {
                	mType = SonyClie;
                }        
            }
            else mType = Palm;
        }
        else mType = Palm;
    }
    else mType = Palm;

	// screen setting 
	SetScreenMode(maxDepth);
	// SetScreenMode(1);
        // initialize color table
    //
    if (is35) {
    	rcolorRed.r = 204;
    	rcolorRed.g = 0;
    	rcolorRed.b = 0;    	
    	// colorRed = WinRGBToIndex(&rcolorRed);
    	
    	rcolorBlue.r = 51;
    	rcolorBlue.g = 0;
    	rcolorBlue.b = 153;
    	// colorBlue = WinRGBToIndex(&rcolorBlue);
    	
    	rcolorBlack.b = rcolorBlack.g = rcolorBlack.r = 0;
    	// colorBlack = WinRGBToIndex(&rcolorBlack);
    	
    	rcolorWhite.r = rcolorWhite.g = rcolorWhite.b = 255;
    	// colorWhite = WinRGBToIndex(&rcolorWhite);
    }
    
    if(mType == Handera330) {
    	VgaSetScreenMode(screenMode1To1, rotateModeNone);
    }
    
    if (mType >= SonyClie) {
		HRWinGetDisplayExtent(hrRefNum, &exx, &exy);
		gOffScreen =
			HRWinCreateOffscreenWindow(hrRefNum, exx, exy, screenFormat, &err);
	}
	else {
		WinGetDisplayExtent(&exx, &exy);
		gOffScreen =
			WinCreateOffscreenWindow(exx, exy, screenFormat, &err);		
	}

   return errNone;
}


/***********************************************************************
 * AppStop
 ***********************************************************************/
static void AppStop(void)
{
    TStarterPreference prefs;
    
    // restore key rates
	KeyRates(true,&gKeyP1,&gKeyP2,&gKeyP3,&gKeyP4);
		
	WinDeleteWindow(gOffScreen, false);
 
	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	MemSet(&prefs,sizeof(prefs),0);


	{
		int i;
		prefs.dbVersion=DB_VERSION;
		prefs.level=gLevel;
		prefs.button=gControlButton;
		for(i=0;i<LEVEL_NUM;++i) {
			prefs.highScore[i]=gHighScore[i];
		}
	}

    if (mType >= SonyClie) {
        HRWinScreenMode(hrRefNum, winScreenModeSetToDefaults,
                        NULL, NULL, NULL, NULL);
        HRClose(hrRefNum);
    }
		
	PrefSetAppPreferences (APP_FILE_CREATOR, APP_PREF_ID, APP_PREF_VERSION, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}

/***********************************************************************
 * PilotMain
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error;
	
	switch (cmd)
	{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

	}
	
	return errNone;
}

